# mysqlbackup v2.2
# Script to make automatic backups of all your mysqldatabases
# Made by fribse at QNAP forum
# Thanks to adnovea, patbaker82, silas, Don for inspiration and help, and 
#           jm45 and bramschats for testing

# USER MODIFYABLE SECTION
OLDBKUP=15                         # Set how many backups to keep
MYSQL_USER="backup"                # User for accessing the databases, password stored in the file /etc/config/backuppass
BKUP_S="Qbackup"                   # Share created for storing backups, HAS to be a share, not just a folder

# Standard commands used in script
RM_C="/bin/rm"
SED_C="/bin/sed"
TAR_C="/bin/tar"
AWK_C="/bin/awk"
GET_C="/sbin/getcfg"
EC_C="/bin/echo"
LOG_C="/sbin/write_log"

# Internal variable setup
ARC=`date +%y%m%d`.tar.gz
MYSQL_P=
MYSQL_C=

error ()
{ $EC_C -e "\nERROR $1\n" ; $LOC_C "MySQL Backup: ERROR $1" 1 ; exit 1 ; }

warn ()
{ $EC_C -e "\nWARNING $1\n" ;  $LOC_C "MySQL Backup: WARNING $1" 2 ; }

info ()
{ $EC_C -e "\nMySQL Backup: $1\n" ; $LOG_C "MySQL Backup: $1" 4 ; } 

# Checking if prerequisites are met
info "STARTED"

if [ -f /etc/config/backuppass ] ; then MYSQL_PASS=`cat /etc/config/backuppass` ; else error "Password file missing, please create a file called backuppass in the folder /etc/config containing only the password" ; fi
BKUP_P=`$GET_C $BKUP_S path -f /etc/config/smb.conf`
if [ $? != 0 ] ; then error "The share $BKUP_S is not found, remember that the destination has to be a share" ; fi
for mysql in /share/HDA_DATA /share/HDB_DATA /share/HDC_DATA /share/HDD_DATA /share/HDE_DATA /share/HDF_DATA /share/HDG_DATA /share/HDH_DATA /share/MD0_DATA; do
  [ -d $mysql/.@mysql ] && MYSQL_P="$mysql/.@mysql"
done
if [ -z $MYSQL_P ] ; then error "MySQL not found." ; fi
if ! [ -d $BKUP_P/mysql ] ; then warn "mysql folder missing under $BKUP_P, it will been created" ; mkdir $BKUP_P/mysql ; if [ $? != 0 ] ; then error "The folder mysql could not be created on the share $BKUP_S" ; fi ; fi
if ! [ -d $BKUP_P/oldmysql ] ; then warn "oldmysql folder missing under the share $BKUP_P, it will been created" ; mkdir $BKUP_P/oldmysql ; if [ $? != 0 ] ; then error "The folder oldmysql could not be created on the share $BKUP_S" ; fi ; fi
for dump in /mnt/ext/opt/mysql /mnt/HDA_ROOT/mysql /mnt/HDB_ROOT/mysql /mnt/HDC_ROOT/mysql /mnt/HDD_ROOT/mysql /mnt/HDE_ROOT/mysql /mnt/HDF_ROOT/mysql /mnt/HDG_ROOT/mysql /mnt/HDH_ROOT/mysql /share/MD0_DATA/.qpkg/Optware; do
  [ -f $dump/bin/mysqldump ] && MYSQL_C="$dump/bin/mysqldump"
done
if [ -z $MYSQL_C ] ; then error "mysqldump command not found. Please use the command 'find / -name mysqldump' and send the result to the author"; fi

# Delete old backups
info "Cleaning out old backups. Keeping the last $OLDBKUP backups"
$RM_C -f `ls -t $BKUP_P/oldmysql/* | $AWK_C 'NR>'$OLDBKUP`
if [ $? != 0 ] ; then error "Ereasing old backups" ; fi

# Compress yesterdays backup to an archive
info "Compressing old backup to $BKUP_P/oldmysql/$ARC"
$TAR_C 2> /dev/null -czvf $BKUP_P/oldmysql/$ARC $BKUP_P/mysql/
if [ $? != 0 ] ; then error "Compressing backup" ; fi
info "Done compressing old backup"
info "Backing up current databases"

# Listing all the databases, and dumping them
$RM_C -f $BKUP_P/mysql/*
find $MYSQL_P -type d | $SED_C 's,^\(.*/\)\?\([^/]*\),\2,' | $SED_C '1d' | while read LINE
do
  set $LINE
  $EC_C "Backing up database $LINE to $BKUP_P/mysql/$LINE.sql\n"
  $MYSQL_C -eqQ -u $MYSQL_USER -p $LINE --password=$MYSQL_PASS --single-transaction > $BKUP_P/mysql/$LINE.sql
  if [ $? != 0 ]; then error "creating new backup when trying to access the database $LINE" ; fi
done
if [ $? = 0 ] ; then info "Backup Successfull" ; fi
